# -*- coding: utf-8 -*-
'''
Created on Aug 31, 2010

@author: Jan H. Meinke
'''

import smmp_p as smmp
import numpy as np
from math import sqrt, cos

def scan():
    """Iterate through all interaction pairs and return lists of 
    interaction partners for each type of interaction.
    
    This implementation is almost equivalent to the Fortran 
    implementation in enyshe.f.
    
    """
    cutoff = 32
    longDistancePartners = {}
    oneFourPartners = {}
    
    EvdW = 0
    Ec = 0
    Ehb = 0
    Etor = 0
    ifivr = smmp.mol_i.ivrml1[0]
    ntlvr = smmp.mol_par.nvr
    i1s = smmp.mol_i.imsml1[0] + smmp.mol_i.nmsml[0]
    # Loop over variables in reverse order.
    for iv in smmp.var_i.iorvr[ifivr + ntlvr - 2:ifivr:-1] - 1:
        ia = smmp.var_i.iatvr[iv] - 1
        ic = smmp.var_i.iclvr[iv] - 1
        if smmp.var_i.ityvr[iv] == 3:
            Etor += smmp.epar_r.e0to[ic] * (1.0 + smmp.epar_r.sgto[ic] *
                cos(smmp.atm_r.toat[ia] * smmp.epar_r.rnto[ic]))
            
        i2s = i1s - 1
        i1s = smmp.var_i.imsvr1[iv - 1]
        for i1, i2 in zip(smmp.var_i.latms1[i1s:i2s] - 1, 
                          smmp.var_i.latms2[i1s:i2s]):
            for i, ity, cqi, xi, yi, zi, ivw1, ivw2, i141, i142 in zip(
                    np.arange(i1, i2), smmp.atm_i.ityat[i1:i2], 
                    smmp.epar_r.conv * smmp.atm_r.cgat[i1:i2],
                    smmp.atm_r.xat[i1:i2], 
                    smmp.atm_r.yat[i1:i2], smmp.atm_r.zat[i1:i2], 
                    smmp.eny_i.ivwat1[i1:i2] - 1, smmp.eny_i.ivwat2[i1:i2],
                    smmp.eny_i.i14at1[i1:i2] - 1,smmp.eny_i.i14at1[i1:i2]):
                longDistancePartners[i] = []
                oneFourPartners[i] = []
                for j, jty, xij, yij, zij, cqj in zip(np.arange(ivw1, ivw2), 
                        smmp.atm_i.ityat[ivw1:ivw2],
                        smmp.atm_r.xat[ivw1:ivw2] - xi, 
                        smmp.atm_r.yat[ivw1:ivw2] - yi,
                        smmp.atm_r.zat[ivw1:ivw2] - zi,
                        smmp.atm_r.cgat[ivw1:ivw2]):
                    if abs(i - j) < cutoff:
                        longDistancePartners[i].append(j)
                    rij2 = xij * xij + yij * yij + zij * zij
                    rij4 = rij2 * rij2
                    rij6 = rij4 * rij2
                    rij = sqrt(rij2)
                    Ec += cqi * cqj / rij
                    if smmp.epar_i.ihbty[ity, jty]:
                        Ehb += smmp.epar_r.ahb[ity, jty] / (rij6 * rij6) -\
                            smmp.epar_r.chb[ity, jty] / (rij6 * rij4)
                    else:
                        EvdW += smmp.epar_r.aij[ity, jty] / (rij6 * rij6) -\
                            smmp.epar_r.cij[ity, jty] / rij
                
                for j in smmp.eny_i.l14at[i141:i142]:
                    jty = smmp.atm_i.ityat[j]
                    xij = smmp.atm_r.xat[j] - xi
                    yij = smmp.atm_r.xat[j] - yi
                    zij = smmp.atm_r.xat[j] - zi
                    rij2 = xij * xij + yij * yij + zij * zij
                    rij4 = rij2 * rij2
                    rij6 = rij4 * rij2
                    rij = sqrt(rij2)
                    Ec += cqi * cqj / rij
                    if smmp.epar_i.ihbty[ity, jty]:
                        Ehb += smmp.epar_r.ahb[ity, jty] / (rij6 * rij6) -\
                            smmp.epar_r.chb[ity, jty] / (rij6 * rij4)
                        longDistancePartners[i].append(j)
                    else:
                        oneFourPartners[i].append(j)
                        EvdW += smmp.epar_r.a14[ity, jty] / (rij6 * rij6) -\
                           smmp.epar_r.cij[ity, jty] / rij
                           
                if not longDistancePartners[i]:
                    del(longDistancePartners[i])
                if not oneFourPartners[i]:
                    del(oneFourPartners[i])

                    
    print EvdW + Ec + Ehb + Etor, EvdW, Ec, Ehb, Etor
    
    return longDistancePartners, oneFourPartners
                 
